@@ -90,7 +90,7 @@ class CompiledSQL(Sql):
     def finalize(self, qb):
         wrapped = self.wrap(qb)
         assert qb.is_root
-        if wrapped.type <= T.primitive:
+        if wrapped.type <= T.primitive and not wrapped.code[0].lower().startswith('select '):
             code = ['SELECT '] + wrapped.code
         else:
             code = wrapped.code
@@ -120,8 +120,7 @@ class CompiledSQL(Sql):
     def compile(self, qb):
         return self
     def finalize_with_subqueries(self, qb, subqueries):
-        assert not subqueries   # TODO
-        return self.compile(qb).finalize(qb)
+        return SqlTree.finalize_with_subqueries(self, qb, subqueries)
 
     def optimize(self):
         if not self.code:
@@ -276,18 +275,17 @@ class Case(SqlTree):
             code += [ " ELSE " ] + self.else_.compile_wrap(qb).code
         return code + [" END "]
 
+_ARRAY_SEP = '||'
 
 @dataclass
 class MakeArray(SqlTree):
     type: Type
     field: Sql
 
-    _sp = "|"
-
     def _compile(self, qb):
         field = self.field.compile_wrap(qb).code
         if qb.target == sqlite:
-            return ['group_concat('] + field + [f', "{self._sp}")']
+            return ['group_concat('] + field + [f', "{_ARRAY_SEP}")']
         elif qb.target == postgres:
             return ['array_agg('] + field + [')']
         elif qb.target == mysql:
@@ -742,6 +740,11 @@ class Subquery(SqlTree):
         fields_str = ["("] + join_comma(fields) + [")"] if fields else []
         return [f"{self.table_name}"] + fields_str + [" AS ("] + query + [")"]
 
+def _enum_is_last(seq):
+    last = len(seq) - 1
+    for i, item in enumerate(seq):
+        yield i == last, item
+
 
 @dataclass
 class Join(TableOperation):
@@ -753,14 +756,18 @@ class Join(TableOperation):
     def _compile(self, qb):
         tables_sql = [t.compile_wrap(qb).code for t in self.tables]
         join_op = ' %s ' % self.join_op.upper()
-        join_sql = join_sep([e for e in tables_sql], join_op)
 
-        if self.conds:
-            conds = join_sep([c.compile_wrap(qb).code for c in self.conds], ' AND ')
-        else:
-            conds = ['1=1']   # Postgres requires ON clause
+        code = [f'SELECT * FROM '] + tables_sql[0]
 
-        return [f'SELECT * FROM '] + join_sql + [' ON '] + conds
+        for is_last, t_sql in _enum_is_last(tables_sql[1:]):
+            code += [join_op] + t_sql + [' ON ']
+
+            if self.conds and is_last:
+                code += join_sep([c.compile_wrap(qb).code for c in self.conds], ' AND ')
+            else:
+                code += ['1=1']   # Postgres requires ON clause
+
+        return code
 
 
 
@@ -884,7 +891,7 @@ def add_one(x):
     return BinOp('+', [x, make_value(1)])
 
 def _quote(target, name):
-    assert isinstance(name, str)
+    assert isinstance(name, str), name
     if target is sqlite:
         return f'[{name}]'
     elif target is mysql or target is bigquery:
@@ -919,8 +926,11 @@ def compile_type_def(state, table_name, table) -> Sql:
             # TODO any column, using projection / get_attr
             if not table.options.get('temporary', False):
                 # In postgres, constraints on temporary tables may reference only temporary tables
-                s = f"FOREIGN KEY({name}) REFERENCES {_quote(target, c.options['name'])}(id)"
-                posts.append(s)
+                rel = c.options['rel']
+                if rel['key']:          # Requires a unique constraint
+                    _tbl_name ,= rel['table'].options['name'].parts   # TODO fix for multiple parts
+                    s = f"FOREIGN KEY({name}) REFERENCES {_quote(target, _tbl_name)}({rel['column']})"
+                    posts.append(s)
 
     if pks:
         names = ", ".join(pks)
@@ -937,7 +947,8 @@ def compile_drop_table(state, table_name) -> Sql:
 @dp_type
 def compile_type(type_: T.t_relation):
     # TODO might have a different type
-    return 'INTEGER'    # Foreign-key is integer
+    #return 'INTEGER'    # Foreign-key is integer
+    return compile_type(type_.elems['item'])
 
 @dp_type
 def compile_type(type_: T.primitive):
@@ -948,7 +959,7 @@ def compile_type(type_: T.primitive):
         'float': "FLOAT",
         'bool': "BOOLEAN",
         'text': "TEXT",
-        't_relation': "INTEGER",
+        # 't_relation': "INTEGER",
         'datetime': "TIMESTAMP",
     }[type_.typename]
     if not type_.maybe_null():
@@ -1047,7 +1058,7 @@ def restructure_result(state, t: T.list[T.union[T.primitive, T.nulltype]], i):
     if state.db.target == mysql:
         res = json.loads(res)
     elif state.db.target == sqlite:
-        res = res.split('|')
+        res = res.split(_ARRAY_SEP)
 
     # XXX hack! TODO Use a generic form to cast types
     if t.elem <= T.int:
